from pathlib import Path
import csv
from datetime import datetime, timedelta

def get_all_data_from_folder(foldername, datatype="txt"):
    """extracts all files from given folder for further processing"""
    script_dir = Path(__file__).resolve().parent

    data_dir = script_dir.parent / f"{foldername}"

    files = list(data_dir.rglob(f"*.{datatype}"))
    
    answers = []
    # only gets first file from files
    with open(files[0], 'r', encoding='utf-8') as file:
        reader = csv.reader(file)
        for row in reader:
            answers.append(row)
    return answers

def process_survey_data(headers, answers):
    """
    Looks at the tags/questions and adds to every question every response of every participant.
    In the end you get a dict with question names as keys and the values are the answers of every participant.
    So the first participant will have the index 0 in every value.
    """
    survey_group_answers = {}
    for responses in answers:
        question_num = 0
        for idx, question in enumerate(headers):
            if "Which text was" in question:
                question_num += 1
            if 3 < idx < 154:
                if f"{question}/{question_num}" not in survey_group_answers:
                    survey_group_answers[f"{question}/{question_num}"] = []
                    survey_group_answers[f"{question}/{question_num}"].append(responses[idx])
                else:
                    survey_group_answers[f"{question}/{question_num}"].append(responses[idx])
            else:
                if f"{question}" not in survey_group_answers:
                    survey_group_answers[f"{question}"] = []
                    survey_group_answers[f"{question}"].append(responses[idx])
                else:
                    survey_group_answers[f"{question}"].append(responses[idx])
    return survey_group_answers

class Proccess_Data(object):
    
    def __init__(self, correct_labels, survey_data, models):
        self.correct_labels = correct_labels
        self.survey_data = survey_data
        self.models = models
    
    @staticmethod
    def calculate_total_time(time_start, time_end):
        total_time = timedelta()
        
        user_time = {}
        for num, (start, end) in enumerate(zip(time_start, time_end), start=1):
            start_time = datetime.strptime(start, "%H:%M:%S")
            end_time = datetime.strptime(end, "%H:%M:%S")
            
            if end_time < start_time:
                continue
            time = end_time - start_time
            
            if time.total_seconds() / 60 < 180:
                total_time += (end_time - start_time)
                user_time[num] = time.total_seconds() // 60
        
        return round((total_time.total_seconds() / 60) / len(time_start), 2) , user_time
    
    def calculate_correct_answers(self, indicies_flag=False, indicies=None):
        correct_total = 0
        incorrect_total = 0
        total = 0
        results = {}
        for num in range(1, len(self.survey_data['Which text was written by the LLM?/1']) + 1):
            all_answers = self.survey_data[f'Which text was written by the LLM?/{num}']
            correct_group = 0
            incorrect_group = 0
            total_group = 0
            for idx, single_answers in enumerate(all_answers):
                if indicies_flag:
                    if idx in indicies:
                        total += 1
                        total_group += 1
                        if single_answers == self.correct_labels[num-1]:
                            correct_group += 1
                            correct_total += 1
                        else:
                            incorrect_total += 1
                            incorrect_group += 1
                else:
                    total += 1
                    total_group += 1
                    if single_answers == self.correct_labels[num-1]:
                        correct_group += 1
                        correct_total += 1
                    else:
                        incorrect_total += 1
                        incorrect_group += 1
            if total_group > 0:
                results[num] = [correct_group, incorrect_group, round(correct_group / total_group, 2)]
            # else:
            #     results[num] = [correct_group, incorrect_group, 0]
        results.setdefault(0, [correct_total, incorrect_total, round(correct_total / total, 2)])
        return results # {survey_num: [correct, false, correct_percentage]}
    
    def compare_groups(self, group):
        if group == "expert":
            output = self.survey_data['Do you study computerlinguistics or work in a similar field?']
            
            yes_indices = []
            for idx in range(len(output)):
                if output[idx] == "Yes":
                    yes_indices.append(idx)
            expert_indices = self.calculate_correct_answers(indicies_flag=True, indicies=yes_indices)
            return expert_indices
        if group == "ai_usage":
            output = self.survey_data['How often do you use AIs?']
            very_often_indices = []
            for idx in range(len(output)):
                if output[idx] == 'Very often (everyday for all kind of things and tasks)':
                    very_often_indices.append(idx)
            usage_indices = self.calculate_correct_answers(indicies_flag=True, indicies=very_often_indices)
            return usage_indices
        if group == "time":
            time_start = self.survey_data['What time is it now?']
            time_end = self.survey_data['Please enter the time.']
            total_time = self.calculate_total_time(time_start, time_end)
            avg_time = total_time[0]
            above_avg = []
            below_avg = []
            for time in total_time[1].items():
                if time[1] > avg_time + 10:
                    above_avg.append(time[0])
                if time[1] < avg_time - 10:
                    below_avg.append(time[0])
            above_avg_indicies = self.calculate_correct_answers(indicies_flag=True, indicies=above_avg)
            below_avg_indicies = self.calculate_correct_answers(indicies_flag=True, indicies=below_avg)
            return above_avg_indicies, below_avg_indicies
                
    def compare_ai(self, correct_percentage):
        gpt2, opt, gpt4o , gpt2_count, opt_count, gpt4o_count = 0, 0, 0, 0, 0, 0
        for ai, correctness in zip(self.models, correct_percentage):
            if ai == "gpt2":
                gpt2 += correctness
                gpt2_count += 1
            elif ai == "opt":
                opt += correctness
                opt_count += 1
            elif ai == "gpt4o":
                gpt4o += correctness
                gpt4o_count += 1
        return {"gpt2": round(gpt2 / gpt2_count, 2), "opt": round(opt / opt_count, 2), "gpt4o": round(gpt4o / gpt4o_count, 2)}
    
    def average_parameter(self, parameter, model_name):
        """Looks at the parameters like coherence, conciseness, creativity and clarity of concept and calculates the average."""
        count, rate = 0, 0
        if model_name != "human":
            for question_num, correct_label in enumerate(self.correct_labels):
                for keys in self.survey_data.keys():
                    if parameter in keys and correct_label in keys:
                        model = self.models[question_num]
                        if model == model_name: 
                            for rating in self.survey_data[keys]:
                                if rating:
                                    rate += int(rating)
                                    count += 1
        if model_name == "human":
            for question_num, correct_label in enumerate(self.correct_labels):
                for keys in self.survey_data.keys():
                    if parameter in keys and correct_label not in keys:
                        model = self.models[question_num]
                        for rating in self.survey_data[keys]:
                            if rating:
                                rate += int(rating)
                                count += 1

        return round(rate / count, 2)
        
if __name__ == "__main__":
    answers = get_all_data_from_folder("results", "csv")
    headers = answers[0]
    only_answers = answers[1:]

    survey_data = process_survey_data(headers, only_answers)

    #print(survey_data.keys())
    
    # correct answers for each survey group (LLM generated text is the correct answer)
    correct_text = ["Text 2", "Text 2", "Text 1", "Text 2", "Text 1", "Text 2",
                    "Text 2", "Text 2", "Text 1", "Text 1", "Text 2", "Text 1",
                    "Text 1", "Text 2", "Text 2", "Text 1", "Text 1", "Text 2",]
    
    models = ["gpt2", "gpt2", "opt", "opt", "gpt4o", "gpt4o",
              "gpt4o", "gpt4o", "opt", "opt", "gpt2", "gpt2",
              "opt", "opt", "gpt4o", "gpt4o", "gpt2", "gpt2"]
    
    evaluator = Proccess_Data(correct_text, survey_data, models)
    #total_correct = evaluator.calculate_correct_answers()
    
    # expert_group = evaluator.compare_groups("expert")
    # ai_usage_group = evaluator.compare_groups("ai_usage")
    # time_group = evaluator.compare_groups("time")
    
    #correct_percentage = [i[2] for i in total_correct.values()] # extracts average percentage of correct answers
    #model_results = evaluator.compare_ai(correct_percentage)

    parameters = ["Coherence", "Conciseness", "Creativity", "Clarity of Concept"]
    models = ["gpt2", "gpt4o", "opt", "human"]
    
    avg_scores = {
    param: {model: evaluator.average_parameter(param, model) for model in models}
    for param in parameters
    }

    


